/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.account.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import se.vgregion.account.services.repository.ExternalUserStructureRepository;
import se.vgregion.account.services.util.StructureQueryUtil;
import se.vgregion.create.domain.ExternalUserStructure;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

public class StructureServiceImpl implements StructureService {

    @Autowired
    private ExternalUserStructureRepository structureRepository;

    @Value("${max.results:10}")
    private int maxResults;

    @Autowired
    private StructureQueryUtil queryUtil;

    @Override
    public Collection<String> search(String query) {
        Set<String> replyList = new TreeSet<String>();

        if (query == null) {
            query = "";
        }
        query = query.toLowerCase();
        String[] queryParts = query.split("/");
        queryParts[queryParts.length - 1] += "%";

        String joinWhereClause = queryUtil.whereClause(queryParts, true);

        List<ExternalUserStructure> result = (List<ExternalUserStructure>) structureRepository
                .findByQuery("select s from ExternalUserStructure s " + joinWhereClause, queryParts);

        int cnt = 0;
        outer:
        for (ExternalUserStructure externalUserStructure : result) {
            String base = queryUtil.resolveBase(externalUserStructure);
            List<ExternalUserStructure> children = (List<ExternalUserStructure>) structureRepository
                    .findByQuery("select s from ExternalUserStructure s join s.parent s2 where s2 = ?1",
                            new Object[]{externalUserStructure});
            if (replyList.add(base)) {
                cnt++;
            }
            if (cnt >= maxResults) {
                break;
            }
            for (ExternalUserStructure child : children) {
                if (replyList.add(base + "/" + child.getName())) {
                    cnt++;
                }
                if (cnt >= maxResults) {
                    break outer;
                }
            }
        }

        return replyList;
    }

    @Override
    @Transactional
    public void storeExternStructurePersonDn(String externStructurePersonDn) {
        String[] parts = externStructurePersonDn.split("/");

        // iterate through parts until an existing parent is found
        ExternalUserStructure parent = null;
        int i = parts.length;
        while (i > 0) {
            parent = existingStructure(Arrays.copyOf(parts, i));
            if (parent != null) {
                break; // existing parent found
            }
            i--;
        }

        // create the children that were not found
        for (int j = i; j < parts.length; j++) {
            ExternalUserStructure child = new ExternalUserStructure();
            child.setParent(parent);
            child.setName(parts[j]);

            structureRepository.persist(child);

            parent = child;
        }
    }

    private ExternalUserStructure existingStructure(String[] parts) {
        String where = queryUtil.whereClause(parts, false);
        List<ExternalUserStructure> result = (List<ExternalUserStructure>) structureRepository
                .findByQuery("select s from ExternalUserStructure s " + where, parts);

        if (result.size() == 1) {
            return result.get(0);
        } else {
            return null;
        }
    }

    @Override
    public void setMaxResults(int maxResults) {
        this.maxResults = maxResults;
    }
}
